# Script ler_faturas_Elektro v1.0 - 16/11/2021
# Elaboracao: Daniel Hoffman
# Testes: Fernando Finger

# Clean residual data
rm(list = ls())
# library need for using pipes
library(dplyr)
# match list for month abreviations
meses <- c("jan", "fev", "mar", "abr", "mai", "jun", "jul", "ago", "set" , "out", "nov" , "dez" )
#########################################################################################
# get .pdf directory - will refresh until selection of any directory
tcltk::tkmessageBox(title = "AVISO!", message = paste("Selecione a pasta raiz que contem os arquivos .PDF: "), icon = "info", type = "ok")
repeat{
    Pasta_Arquivos_PDF <- tcltk::tk_choose.dir("Escolher pasta para exploracao")
    if(!is.na(Pasta_Arquivos_PDF)){
        break
    }
}
# build filter for Excel files
filters <- matrix(c("XLSX files", ".xlsx"), 2, 2, byrow = TRUE)
# get xlsx file path though canceladion/non selection will not cause refresh!
tcltk::tkmessageBox(title = "AVISO!", message = paste("Selecione o arquivo de dados ja existente. Caso nao exista, clique em cancelar."), icon = "info", type = "ok")
repeat{
    Excel_Output <- tcltk::tk_choose.files(filter = filters)
	# if user cancels/closes screen set variable for later use
    if(length(Excel_Output) == 0){
	    Excel_Output <- "nada"
	    break
    }  
	# if user selected a file check if file is in use and abort script in this case
    if(!is.na(Excel_Output)){
    	# check by searching for files with same name in directory (bkp file should have only diferent preceding characters 
        if(sum(stringr::str_detect(list.files(dirname(Excel_Output)), 
                                   stringr::str_remove(Excel_Output, paste0(dirname(Excel_Output),"/")))) != 1){
			# send out message
            tcltk::tkmessageBox(title = "PROBLEMA ;(", message = "Ou Planilha esta aberta ou nao existe?!", icon = "info", type = "ok")
            # causes script to fail
            UNDECLARED()
        }
        break
    }
}
tryCatch(
	expr = {
        # check if file exists (nada wont)
        if(file.exists(Excel_Output)){
            # check if excel file has at least on worksheet with fatura data
            if(sum(readxl::excel_sheets(path = Excel_Output) %in% c("fat_A", "fat_B")) > 0){
		        # check if sheet for fatura type A exists and read it else declare variable as NULL
		        if(sum(readxl::excel_sheets(path = Excel_Output) == "fat_A") == 1){
			        read_data_A <- openxlsx::read.xlsx(Excel_Output, sheet = "fat_A") %>%
			                           dplyr::mutate(Referencia = as.Date(Referencia, origin = "1899-12-30"),
									                 LeitAnterior = as.Date(LeitAnterior, origin = "1899-12-30"),
							                         LeitAtual = as.Date(LeitAtual, origin = "1899-12-30"))
			        read_files_A <- unique(read_data_A$.id)
		        }else{
			        # declare empty variable for later use
			        read_data_A <- NULL
			        read_files_A <- NULL
		        }
		        # check if sheet for fatura type B exists and read it else declare variable as NULL
		        if(sum(readxl::excel_sheets(path = Excel_Output) == "fat_B") == 1){
			        read_data_B <- openxlsx::read.xlsx(Excel_Output, sheet = "fat_B") %>%
			                           dplyr::mutate(Referencia = as.Date(Referencia, origin = "1899-12-30"),
									                 LeitAnterior = as.Date(LeitAnterior, origin = "1899-12-30"),
							                         LeitAtual = as.Date(LeitAtual, origin = "1899-12-30"))
			        read_files_B <- unique(read_data_B$.id)
		        }else{
			        # declare empty variable for later use
			        read_data_B <- NULL
			        read_files_B <- NULL
		        }
		        # combine all paths to one vector for later use
	            read_files <- c(read_files_A, read_files_B)
            }else{
                # msg where new output file will be created and create path plus empty variables
                tcltk::tkmessageBox(title = "AVISO!", message = paste("Sera criado novo arquivo em:\n", Pasta_Arquivos_PDF), icon = "info", type = "ok")
	            Excel_Output <- paste0(Pasta_Arquivos_PDF, "/output.xlsx")
                read_data_A <- NULL
	            read_data_B <- NULL
                read_files <- NULL
            }
		}else{
	        Excel_Output <- paste0(Pasta_Arquivos_PDF, "/output.xlsx")
            # msg where new output file will be created and create path plus empty variables
            tcltk::tkmessageBox(title = "AVISO!", message = paste("Sera criado novo arquivo em:\n", Excel_Output), icon = "info", type = "ok")
            read_data_A <- NULL
	        read_data_B <- NULL
            read_files <- NULL
        }
	},error = function(e){ 
    msgBox <- tcltk::tkmessageBox(title = "PROBLEMA ;(",
                                  message = "Algo no arquivo Excel parece estar errado - tente remover-lo e execute a leitura novamente.", icon = "info", type = "ok")
    UNDECLARED()
  }
)
# one could check if the excel files have the needed columns names and data as this could generate and error
#########################################################################################
# list all .pdf files wih full names and directories
files <- list.files(Pasta_Arquivos_PDF, full.names = TRUE, recursive = TRUE, include.dirs = TRUE)
# reduce to only .pdf files with path
pdf_files <- data.frame(path = files[stringr::str_detect(files, pattern = "\\.pdf$")]) 
# files that have not previously been read
pdf_files_to_read <- pdf_files %>% 
    dplyr::filter(!path %in% read_files) %>% 
    dplyr::pull(path) %>%
	stringr::str_replace("//", "/")
# avisar sobre pre leitura
tcltk::tkmessageBox(title = "AVISO!", message = paste("Foram encontrados:", nrow(pdf_files), "arquivos de PDF!\nSendo", length(pdf_files_to_read), "novos!"), icon = "info", type = "ok")
# function to read in file, read lines and drop leading and trailing blanks
read_pdfs <- function(file){
  pdf_text <- pdftools::pdf_text(file)[[1]] %>% 
        readr::read_lines() %>%
        trimws()
    return(pdf_text) 
}
# read in all files... possibly need to be more error prone
read_pdfs <- purrr::possibly(read_pdfs,"0000000000")
pdf_texts <- purrr::map(pdf_files_to_read, ~read_pdfs(.x)) 
# pdf function to see if it is type A, B or Segunda Via (if all zero then none of the prior)
judge_pdfs <- function(pdf_text){  
    # check what typ of pdf it is
    val <- data.frame(A = sum(stringr::str_detect(stringr::str_to_lower(pdf_text), "classifica.*:")),
                      B = sum(stringr::str_detect(stringr::str_to_lower(pdf_text), "controle n")),
                      S = sum(stringr::str_detect(stringr::str_to_lower(pdf_text), "segunda via")))
    return(val)
}
# make custom function failure proof (in case of all 0)
judge_pdfs <- purrr::possibly(judge_pdfs, otherwise = data.frame(B = 0, A = 0, S = 0))
# apply rename and bind to df
ret <- purrr::map(pdf_texts, ~judge_pdfs(.x))
names(ret) <- pdf_files_to_read
valuation <- plyr::ldply(ret)
# all segunda vias
segunda_via <- data.frame(path = valuation[valuation$S != 0, 1]) %>%
    dplyr::mutate(motivo = "segunda via")
# not relevant pdfs
nao_e_fatura <- data.frame(path = valuation[valuation$A == 0 & valuation$B == 0, 1]) %>%
    dplyr::mutate(motivo = "nao eh fatura")
# all relevant pdfs
towork <- valuation[valuation$S == 0 & (valuation$A != 0 | valuation$B != 0), ]
# all group A faturas
grupo_a <- towork[towork$A != 0, 1]
# all group A faturas
grupo_b <- towork[towork$A == 0, 1]
# inform numerical resume
tcltk::tkmessageBox(title = "AVISO!", message = paste("Faturas grupo A:", length(grupo_a), "\nFaturas grupo B:", length(grupo_b),"\nSegunda Via:", nrow(segunda_via), "\nNao sao Faturas:", nrow(nao_e_fatura)), icon = "info", type = "ok")
# abort by calling undeclared variable in case no new work is found (preserve last run with usefull data)
if(nrow(towork) == 0){
	tcltk::tkmessageBox(title = "AVISO!", message = paste("Nenhuma fatura nova para processar!"), icon = "info", type = "ok")
    UNDECLARED
}
tcltk::tkmessageBox(title = "AVISO!", message = paste("Por favor, clique OK e aguarde. Voce sera notificado assim que a leitura terminar!"), icon = "info", type = "ok")
#########################################################################################
# helper function to detect, clean and separte regex data
get_regex <- function(pdf, my_regex){
    ret <- tidyr::extract(pdf, col = "lines", into = "referencia", regex = my_regex, remove = FALSE) %>% 
        dplyr::filter(!is.na(referencia)) %>%
        dplyr::pull(referencia)
    return(ret)
}
# function to detect and label relevant info
get_descprod_with_info_ELEKTRO_A <- function(data, sear, info){
    data %>% 
        dplyr::filter(stringr::str_detect(stringr::str_to_lower(DescProd), pattern = sear)) %>% 
	    dplyr::mutate(Msg = info) %>%
	    dplyr::select(.id, Referencia, UC, Msg, VlrTot)			
}
# 
get_column <- function(pdfs, table_data, x1, x2, varname, x0, x01){

	ret <- table_data %>%
        dplyr::filter(between(x, x0, x01)) %>% 
        dplyr::summarise(Descricao = paste(text, collapse = "~")) %>%
        # Referencia
        dplyr::full_join(table_data %>% 
                             dplyr::filter(x > x1 & x < x2) %>%
		                     dplyr::mutate(text = trimws(stringr::str_remove_all(text, pattern = "~"))) %>%
		                     dplyr::group_by(y) %>%
		                     dplyr::select(yn = y, !!varname := text) %>%
		                     dplyr::slice_head(n = 1), 
						by = character()) %>% 
	    dplyr::group_by(y) %>% 
	    dplyr::mutate(dif = abs(y - yn)) %>% 
	    dplyr::slice_min(dif, n = 1) %>% 
	    dplyr::filter(dif < 4) %>% 
	    dplyr::select(y, !!varname)
		
	return(ret)
}


#########################################################################################
tryCatch(
	expr = {
        # only process type A if any is there to read
        if(length(grupo_a) != 0){
            # main function to extract data
            explore_fatura_a <- function(arquivo){
                # read lines to tibble
				pdfs <- pdftools::pdf_data(arquivo)
				
			    # filter table region from first page of pdf doc
        	    table_data <- pdfs[[1]] %>% 
            	    dplyr::filter(y > 150 & y < 400) %>% 
	        	    dplyr::group_by(y) %>%
            	    dplyr::arrange(y, x) %>%
				    dplyr::mutate(text = trimws(text))
  
        	    d <- table_data %>%
            	    dplyr::filter(x < 135)%>% 
            	    dplyr::summarise(DescProd = paste(text, collapse = "~")) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 135, 170, "LeitAnt", 0, 135)) %>% #135, 170
	        	    # Qtd. Reg.
            	    dplyr::left_join(get_column(pdfs, table_data, 170, 210, "LeitAtu", 0, 135)) %>% #170, 210
	        	    # Qtd. Fat.
            	    dplyr::left_join(get_column(pdfs, table_data, 210, 250, "ConstMul", 0, 135)) %>% #210, 250
	        	    # Unidade
            	    dplyr::left_join(get_column(pdfs, table_data, 250, 270, "QtdReg", 0, 135)) %>% #250, 270
	        	    # tarifa com tributos
            	    dplyr::left_join(get_column(pdfs, table_data, 270, 310, "QtdRes", 0, 135)) %>% #270, 310
	        	    # vlr Tot
            	    dplyr::left_join(get_column(pdfs, table_data, 310, 350, "QtdFat", 0, 135)) %>% #310, 350
	        	    # base ICMS
            	    dplyr::left_join(get_column(pdfs, table_data, 350, 390, "TariFor", 0, 135)) %>% #350, 390
	        	    # aliqota ICMS
            	    dplyr::left_join(get_column(pdfs, table_data, 390, 410, "VlrFor", 0, 135)) %>% #390, 410
	        	    # VlrICMS
            	    dplyr::left_join(get_column(pdfs, table_data, 410, 460, "BCImp", 0, 135)) %>% #410, 460 ALTERADO
	        	    # base pis & confins
            	    dplyr::left_join(get_column(pdfs, table_data, 460, 490, "AliqICMS", 0, 135)) %>% #460, 490 ALTERADO
	        	    # VlrICMS
            	    dplyr::left_join(get_column(pdfs, table_data, 490, 520, "VlrICMS", 0, 135)) %>% #490, 520
	        	    # VlrICMS
            	    dplyr::left_join(get_column(pdfs, table_data, 530, 550, "VlrTot", 0, 135)) 

		        d <- d %>%
				    dplyr::select(-y) %>%
			        # perform operations on all columns by condition or position to detect all numbers correctly
		            dplyr::mutate(across(4:last_col(), ~stringr::str_remove_all(.x, "\\.")),
						          across(4:last_col(), ~stringr::str_replace(.x, ",", "\\.")),
						          across(where(is.numeric), ~as.character(.x)),
					              across(2:last_col(), ~readr::parse_number(.x)),
						          across(2:last_col(), ~ifelse(is.na(.x), 0, .x)))

                # read lines to tibble
                pdf <- tibble::tibble(lines = pdftools::pdf_text(arquivo) %>% 
                    readr::read_lines()) %>% 
                   # include row number and set tab TRUE if line starts with 4 dgts and space
                   dplyr::mutate(rn = dplyr::row_number(),
                                 tab = stringr::str_detect(lines, pattern = "^\\d{4}\\s+")) 

                # get specific info by regex with custom function
                UC <- trimws(pdf[1,1])
                Referencia <- trimws(get_regex(pdf, "\\s(.{1,8}[A-Za-z]{1,}/\\d{4})"))
                LeitAnterior <- get_regex(pdf, "Leitura Anterior:\\s*(\\d{2}/\\d{2}/\\d{4})") %>%
		            lubridate::dmy()
                LeitAtual <- get_regex(pdf, "Leitura Atual:\\s*(\\d{2}/\\d{2}/\\d{4})") %>%
		            lubridate::dmy()
                DiasPeriodo <- get_regex(pdf, "Dias do Per.*odo:\\s*(\\d{1,})") %>%
		            as.numeric()
                DemandaContratada <- get_regex(pdf, "(\\d*,*\\d+)\\s+\\d*,*\\d+%$") %>%
		            as.numeric()
                # build output data frame
                DadosTMP <-  d %>%
                    dplyr::mutate(UC = UC,
                                  Referencia = Referencia,
                                  LeitAnterior = LeitAnterior,
                                  LeitAtual = LeitAtual,
                                  DiasPeriodo = DiasPeriodo,
                                  DemandaContratada = DemandaContratada) # Cria o vetor temporario com UC e mes de referencia
			    # declare function output explicitely
                return(DadosTMP)
            }     
            # make it error proof and the error recognizable in result data
            explore_fatura_a <- purrr::possibly(explore_fatura_a, otherwise = data.frame(DescProd = "0",
                                                                                         LeitAnt = NA,
                                                                                         LeitAtu = NA,
                                                                                         ConstMul = NA,
                                                                                         QtdReg = NA,
                                                                                         QtdRes = NA,
                                                                                         QtdFat = NA,
                                                                                         TariFor = NA,
                                                                                         VlrFor = NA,
                                                                                         BCImp = NA,
                                                                                         AliqICMS = NA,
                                                                                         VlrICMS = NA,
                                                                                         VlrTot = NA,
                                                                                         UC = NA,
                                                                                         Referencia = NA,
                                                                                         LeitAnterior = NA,
                                                                                         LeitAtual = NA,
                                                                                         DiasPeriodo = NA,
                                                                                         DemandaContratada = NA))
            # apply function on vector
            aret <- purrr::map(grupo_a, ~explore_fatura_a(.x))
	        # name list result
            names(aret) <- grupo_a
	        # bind to ty
	        aret <- plyr::ldply(aret) %>%
				dplyr::mutate(Referencia = lubridate::my(paste(match(stringr::str_to_lower(stringr::str_sub(Referencia, start = 1, end = 3)), meses), "-", stringr::str_sub(Referencia, start = 5))),
				              CCI = stringr::str_extract(DescProd, "^\\d{4}"),
							  DescProd = trimws(stringr::str_replace_all(stringr::str_remove(DescProd, "^\\d{4}"), pattern = "~", replacement = " ")),
							  AliqICMS = AliqICMS/100) %>%
				dplyr::relocate(CCI, .before = DescProd)
            # if no prior data exists create empty df	
	        if(is.null(read_data_A)){
	            read_data_A <- aret[0, ] 
	        }
	        # detect process errors
            erro_de_proces_a <- data.frame(path = aret[aret$DescProd == "0", 1]) %>%
                dplyr::mutate(motivo = "erro fatura A") %>%
		        dplyr::filter(!is.na(path))
	        # remove process errors
	        aret <- aret %>%
	            dplyr::mutate(AliqICMS = AliqICMS / 100) %>%
	            dplyr::filter(DescProd != "0" & !is.na(VlrTot)) 
	        # secure against all error result
	        if(nrow(aret) == 0){
	            aret <- read_data_A[0, ] 
	        }
			# data verification	current data detect faturation type
			aavisos1 <- get_descprod_with_info_ELEKTRO_A(aret, sear = "consumo reat.fora ponta te", info = "CONSUMO REATFORA PONTA TE kWh") %>%
			    dplyr::union(get_descprod_with_info_ELEKTRO_A(aret, sear = "consumo reat.ponta te", info = "CONSUMO REATPONTA TE kWh")) %>%
			    dplyr::union(get_descprod_with_info_ELEKTRO_A(aret, sear = "exced de demanda reativa tusd", info = "EXCED DE DEMANDA REATIVA TUSD k")) %>%
			    dplyr::union(get_descprod_with_info_ELEKTRO_A(aret, sear = "demanda ultrap. tusd kw", info = "DEMANDA ULTRAP TUSD kW")) %>%
			    dplyr::union(get_descprod_with_info_ELEKTRO_A(aret, sear = "multa conta anterior.+", info = "MULTA CONTA ANTERIOR"))			
	        # bind old and new data
            aret <- aret %>%
		        dplyr::union_all(read_data_A) 
			# data verification	current data get unit with growth in last 3
			aavisos2 <- aret %>% 
				    dplyr::group_by(UC, Referencia) %>% 
					dplyr::summarize(QtdReg = sum(QtdReg, na.rm = TRUE)) %>% 
					dplyr::arrange(UC, Referencia) %>% 
					dplyr::slice_max(order_by = Referencia, n = 3) %>% 
					dplyr::mutate(dif = QtdReg - lead(QtdReg)) %>%
					dplyr::filter(dif > 0) %>%
					dplyr::count() %>%
					dplyr::filter(n == 2) %>%
					dplyr::transmute(Grupo = "A",
					                 UC = as.character(UC))		
	        # get fatura summary errors
            erro_vlr_total_a <- aret %>% 
    	        dplyr::filter(DescProd != "Total") %>% 
		        dplyr::group_by(UC, Referencia, .id) %>%
		        dplyr::summarize(SomTot = sum(VlrTot)) %>%
		        dplyr::ungroup() %>% 
		        dplyr::inner_join(aret %>% 
                          	          dplyr::filter(DescProd == "Total") %>%
	                      	          dplyr::select(UC, Referencia, VlrTot), by = c("UC", "Referencia")) %>%
		        dplyr::filter(abs(as.numeric(VlrTot) - as.numeric(SomTot))>.001) %>%
		        dplyr::transmute(path = .id,
	                 	         motivo = "erro valor total")
        }else{
	        # set new data equal to old data as there no new fatura of this type
            aret <- read_data_A
	        erro_vlr_total_a <- NULL
	        erro_de_proces_a <- NULL
        }
	},error = function(e){ 
    msgBox <- tcltk::tkmessageBox(title = "PROBLEMA ;(",
                                  message = "Algo causou falha critica no processamento das faturas do tipo A", icon = "info", type = "ok")
    UNDECLARED()
  }
)
####################################################################
# only process type B if any is there to read
tryCatch(
	expr = {
        if(length(grupo_b) != 0){
        # main function to extract data
            explore_fatura_b <- function(arquivo){	
			
				pdfs <- pdftools::pdf_data(arquivo)
			    # filter table region from first page of pdf doc
                table_data <- pdfs[[1]] %>% 
                    dplyr::filter(y > 160 & y < 400 & x > 290) %>% 
                    dplyr::group_by(y) %>%
                    dplyr::arrange(y, x) %>%
                    dplyr::mutate(text = trimws(text))
  
        	    d <- table_data %>%
            	    dplyr::filter(between(x, 290, 370)) %>% 
            	    dplyr::summarise(DescProd = paste(text, collapse = "~")) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 370, 400, "Quantidade", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 400, 430, "TariFor", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 430, 460, "VlrFor", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 460, 480, "BCImp", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 480, 500, "AliqICMS", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 500, 530, "VlrICMS", 290, 370)) %>%
	        	    # Referencia
            	    dplyr::left_join(get_column(pdfs, table_data, 530, 560, "VlrTot", 290, 370))
				
				d <- d %>%
				    dplyr::select(-y) %>%
			        # perform operations on all columns by condition or position to detect all numbers correctly
		            dplyr::mutate(across(where(is.character), ~stringr::str_remove_all(.x, "\\.")),
					        	  across(where(is.character), ~stringr::str_replace(.x, ",", "\\.")),
						          across(where(is.numeric), ~as.character(.x)),
					              across(2:last_col(), ~readr::parse_number(.x)),
						          across(2:last_col(), ~ifelse(is.na(.x), 0, .x)))	
					
		        pdf <- tibble::tibble(lines = pdftools::pdf_text(arquivo) %>% 
                                                  readr::read_lines()) %>% 
                    # include row number and set tab TRUE if line starts with 4 dgts and space
                    dplyr::mutate(rn = dplyr::row_number(),
					        	  # set 0 or 1 if pattern of "space, 4 digits, one or more space and any letter OR a space followes by total and one or more spaces" can be detected
                                  tab = stringr::str_detect(stringr::str_to_lower(lines), pattern =  "(\\s{2,}\\d{4}\\s+[a-zA-Z])|(\\s+total\\s+)"),
						          tab = ifelse(rn > 50, FALSE, tab),
						          loc = stringr::str_locate(stringr::str_to_lower(lines), pattern =  "(\\s{2,}\\d{4}\\s+[a-zA-Z])|(\\s+total\\s+)")) %>% 
			        dplyr::filter(nchar(lines) != 0) %>% 
			        dplyr::mutate(rn = dplyr::row_number())				  
		       
		        # get specific info by regex with custom function
                UC <- trimws(pdf[1,1])
                Referencia <- trimws(get_regex(pdf, "\\s(.{1,8}[A-Za-z]{1,}/\\d{4})"))
		        datehelp <- pdf %>%
                    dplyr::mutate(new = stringr::str_match(lines, "\\s*(\\d{2}/\\d{2}/\\d{4})")) %>%
                    dplyr::filter(!is.na(new) & rn > 10)
		        LeitAnterior <- lubridate::dmy(datehelp$new[1,2])
		        LeitAtual <- lubridate::dmy(datehelp$new[2,2])
                DiasPeriodo <- pdf %>%
                    dplyr::mutate(new = stringr::str_match(lines, pattern = "\\s*\\d{2}/\\d{2}/\\d{4}\\s+(\\d*\\.*\\d*)")) %>%
                    dplyr::filter(!is.na(new) & rn > 10) %>%
			        dplyr::pull(new) %>% 
			        .[1,2] %>%
			        as.numeric()
		        # build output data frame
                DadosTMP <-  d %>%
                    dplyr::mutate(UC = UC,
                                  Referencia = Referencia,
                                  LeitAnterior = LeitAnterior,
                                  LeitAtual = LeitAtual,
                                  DiasPeriodo = DiasPeriodo)
		        # declare function output explicitely
		        return(DadosTMP)
	        }
            # make it error proof and the error recognizable in result data
            explore_fatura_b <- purrr::possibly(explore_fatura_b, otherwise = data.frame(DescProd = "0",
                                                                                         Quantidade = NA,
                                                                                         TariFor = NA,
                                                                                         VlrFor = NA,
                                                                                         BCImp = NA,
                                                                                         AliqICMS = NA,
                                                                                         VlrICMS = NA,
                                                                                         VlrTot = NA,
                                                                                         UC = NA,
                                                                                         Referencia = NA,
                                                                                         LeitAnterior = NA,
                                                                                         LeitAtual = NA,
                                                                                         DiasPeriodo = NA))
	        # apply custom function on vector
            bret <- purrr::map(grupo_b, ~explore_fatura_b(.x))
	        # name list result
            names(bret) <- grupo_b
	        # bind to ty
	        bret <- plyr::ldply(bret) %>%
				dplyr::mutate(Referencia = lubridate::my(paste(match(stringr::str_to_lower(stringr::str_sub(Referencia, start = 1, end = 3)), meses), "-", stringr::str_sub(Referencia, start = 5))),
				              AliqICMS = AliqICMS/100,
							  DescProd = trimws(stringr::str_replace_all(stringr::str_remove(DescProd, "^\\d{4}"), pattern = "~", replacement = " ")))
			    
            # if no prior data exists create empty df	
	        if(is.null(read_data_B)){
	            read_data_B <- bret[0, ] 
	        }
            # detect process errors
            erro_de_proces_b <- data.frame(path = bret[bret$DescProd == "0", 1]) %>%
                dplyr::mutate(motivo = "erro fatura B") %>%
		        dplyr::filter(!is.na(path))
	        # remove process error from fatura data output
	        bret <- bret %>%
	            dplyr::filter(DescProd != "0" & !is.na(VlrTot)) 
	        # secure against all error result
	        if(nrow(bret) == 0){
	            bret <- read_data_B[0, ] 
	        }
	        # remove process error from fatura data output
	        bret <- bret %>%
                dplyr::mutate(AliqICMS = AliqICMS / 100) %>%
		        dplyr::union_all(read_data_B) 
						# data verification	current data get unit with growth in last 3
			bavisos2 <- bret %>% 
				    dplyr::group_by(UC, Referencia) %>% 
					dplyr::summarize(Quantidade = sum(Quantidade, na.rm = TRUE)) %>% 
					dplyr::arrange(UC, Referencia) %>% 
					dplyr::slice_max(order_by = Referencia, n = 3) %>% 
					dplyr::mutate(dif = Quantidade - lead(Quantidade)) %>%
					dplyr::filter(dif > 0) %>%
					dplyr::count() %>%
					dplyr::filter(n == 2) %>%
					dplyr::transmute(Grupo = "B",
					                 UC = as.character(UC))		
	        # get fatura summary errors
	        erro_vlr_total_b <- bret %>% 
                dplyr::filter(DescProd != "Total") %>% 
	            dplyr::group_by(UC, Referencia, .id) %>%
            	dplyr::summarize(SomTot = sum(VlrTot)) %>%
            	dplyr::ungroup() %>%
	            dplyr::inner_join(bret %>% 
                                      dplyr::filter(DescProd == "Total") %>%
	                                  dplyr::select(UC, Referencia, VlrTot), by = c("UC", "Referencia")) %>%
	            dplyr::filter(abs(as.numeric(VlrTot) - as.numeric(SomTot))>.001) %>%
	            dplyr::transmute(path = .id,
	                             motivo = "erro valor total")
        }else{
	        # set new data equal to old data as there no new fatura of this type
            bret <- read_data_B
	        erro_vlr_total_b <- NULL
	        erro_de_proces_b <- NULL
        }
	},error = function(e){ 
    msgBox <- tcltk::tkmessageBox(title = "PROBLEMA ;(",
                                  message = "Algo causou falha critica no processamento das faturas do tipo B", icon = "info", type = "ok")
    UNDECLARED()
  }
)
# data combination and export
tryCatch(
	expr = {		
        # create wb object
        wb <- openxlsx::createWorkbook()
		# create worksheets
        openxlsx::addWorksheet(wb, "novidades")
		        # Create a percent style
        pct <- openxlsx::createStyle(numFmt="0%")
        # add percentage style to columns on specific sheets
        if(!is.null(aret)){
            openxlsx::addWorksheet(wb, "fat_A")
            openxlsx::writeData(wb, "fat_A", aret)
			openxlsx::addStyle(wb, "fat_A", style = pct, rows = 1:nrow(aret), cols = 13, gridExpand = TRUE)
		}
        if(!is.null(bret)) {
            openxlsx::addWorksheet(wb, "fat_B")
            openxlsx::writeData(wb, "fat_B", bret)	
            openxlsx::addStyle(wb, "fat_B", style = pct, rows = 1:nrow(bret), cols = 7, gridExpand = TRUE)
        }
		
        openxlsx::addWorksheet(wb, "avisos")
		openxlsx::addWorksheet(wb, "crescente")
        # write data to wb object by name
        openxlsx::writeData(wb, "novidades", rbind(segunda_via, nao_e_fatura, erro_de_proces_a, erro_de_proces_b, erro_vlr_total_a, erro_vlr_total_b) %>%
                             dplyr::select(motivo, path))

		if(exists("aavisos1")) openxlsx::writeData(wb, "avisos", aavisos1)
		if(exists("aavisos2") & exists("bavisos2")){
			openxlsx::writeData(wb, "crescente", dplyr::union(aavisos2, bavisos2))
		} else if (exists("aavisos2") & !exists("bavisos2")) {
			openxlsx::writeData(wb, "crescente", aavisos2)
		} else if (!exists("aavisos2") & exists("bavisos2")) {
			openxlsx::writeData(wb, "crescente", bavisos2)
		}
        # save object to excel file
        openxlsx::saveWorkbook(wb, Excel_Output, overwrite = TRUE)
        # final message: if you got here all went well
        tcltk::tkmessageBox(title = "SOLUCIONADO :)", message = paste("O arquivo output foi criado ou atualizado!", Excel_Output), icon = "info", type = "ok")
	},error = function(e){ 
    msgBox <- tcltk::tkmessageBox(title = "PROBLEMA ;(",
                                  message = "A construcao do resumo ou exportacao causaram o erro", icon = "info", type = "ok")
    UNDECLARED()
  }
)